#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <LPC23xx.h>

#define extern            // Keil: Line added for modular project management

#include "type.h"

#include "easyweb.h"

#include "EMAC.h"
#include "tcpip.h"
#include "udp.h"
#include "i2c.h"
#include "bmp085.h"
#include "uart.h"
#include "fio.h"

#include "webpage.h"                             // webside for our HTTP server (HTML)

#define HTTPC_FREE       0x00 
#define HTTPC_SEND_QUERY 0x01
#define HTTPC_RECV_ACK   0x02
#define HTTPC_END        0x03

char WebQuery[]     = "GET http://www.igorkov.org/XXX/add.php?file=test&data=LPC2378 HTTP/1.0\r\n\r\n";
char WebQueryTemp[] = "GET http://www.igorkov.org/XXX/add.php?file=temp1&data=%d.%d;%d HTTP/1.0\r\n\r\n";
char HTTPQuery[512];

volatile unsigned long timer = 0;

void EventClockHandler(void) __irq
{
  timer++;                                  // upper 16 bits of initial sequence number
  T1IR        = 1;                               // Clear interrupt flag
  VICVectAddr = 0;                               // Acknowledge Interrupt
}

void wait_timer(int val)
{
	unsigned long start = timer;
	while ((timer-start)<val);
}

char *UARTInited = "UART Init Ok!\r\n";
char *UARTConnect = "Connect to server!\r\n";
char *UARTDisConnect = "Close connection!\r\n";
char DataPrintf[256];

int main(void)
{
	int n;
	int StateMachine = 0;
		int32_t temp = 0;
		int32_t press = 0;

	
	for (n=0;n<100000;n++)
	{
		//__NOP();
	}

	if ( I2CInit( (DWORD)I2CMASTER ) == FALSE )	/* initialize I2c */
	{
		while ( 1 );				/* Fatal error */
	}

	GPIOInit( 2, FAST_PORT, DIR_OUT );
	FIO2CLR = 0x000000FF;

	UARTInit(0, 115200, 48000000);	/* baud rate setting */
	UARTSend( 0, (BYTE *)UARTInited, strlen(UARTInited) );

	T1MR0 = 12000000/100;                        // 0.1s
	T1MCR = 3;                                  // Interrupt and Reset on MR0
	T1TCR = 1;                                  // Timer1 Enable
	VICVectAddr5 = (unsigned int)EventClockHandler;// set interrupt vector in 4
	VICIntEnable = 0x00000020;                  // Enable Timer1 Interrupt

	bmp085_init();

	//InitOsc();              // Keil: No oscillator initialization necessary at this time.
	//InitPorts();			  // Keil: No port initialization necessary at this time.

	TCPLowLevelInit();

	//UDP_InitReceiver();
	//UDP_Init_MSFS_Array();

	HTTPStatus = 0;                                // clear HTTP-server's flag register

#if 1
	while (1)
	{
		unsigned long start;
		// 100ms
		switch (StateMachine)
		{
		case 0:
			temp = 0;
			press= 0;
			bmp085_start_temp();
			wait_timer(2);
			bmp085_get_temp(&temp);
			wait_timer(2);
			bmp085_start_baro(2);
			wait_timer(10);
			bmp085_get_baro(&press);
			StateMachine = 1;
			break;
		case 1:
			sprintf(HTTPQuery, WebQueryTemp, temp/10, (temp<0)?((-temp)%10):(temp%10), press);
			UARTSend( 0, (BYTE *)HTTPQuery, strlen(HTTPQuery) );
			StateMachine = 2;
			break;
		case 2:
			// show P2.0 enter
			FIO2SET = 1 << 0;
			UARTSend( 0, (BYTE *)UARTConnect, strlen(UARTConnect) );
			// Ip: 
			RemoteIP[0] = REMIP_1 + (REMIP_2 << 8);
			RemoteIP[1] = REMIP_3 + (REMIP_4 << 8);
			// Ports:
			TCPLocalPort  = 2025;
			TCPRemotePort = 80;
			// Connect:
			TCPActiveOpen();
			start = timer;
			HTTPStatus = HTTPC_SEND_QUERY;
			StateMachine = 3;
			break;
		case 3:
			if (HTTPStatus==HTTPC_FREE)
			{
				TCPClose();
				StateMachine = 5;
				break;
			}
			if ((timer-start)>1000)
			{
				sprintf(DataPrintf, "Timeout: %d, %d\n", SocketStatus, HTTPStatus);
				UARTSend( 0, (BYTE *)DataPrintf, strlen(DataPrintf) );
				TCPClose();
				start = timer;
				StateMachine = 4;
				break;
			}
			break;
		case 4:
			if (TCPStateMachine == CLOSED)
			{
				StateMachine = 5;
				break;
			}
			if ((timer-start)>5000)
			{
				TCPReset();
				StateMachine = 5;
				break;
			}
			break;
		case 5:
			FIO2CLR = 1 << 0;
			UARTSend( 0, (BYTE *)UARTDisConnect, strlen(UARTDisConnect) );
			start = timer;
			StateMachine = 6;
			break;
		case 6:
			//delay 10s
			if ((timer-start)>1000)
				StateMachine = 0;
			break;
		default:
			StateMachine = 0;
			break;
		}
		DoNetworkStuff();
		HTTPClient();
	}
#else
	TCPLocalPort = TCP_PORT_HTTP;                  // set port we want to listen to
  
	while (1)                                      // repeat forever
	{
		if (!(SocketStatus & SOCK_ACTIVE)) TCPPassiveOpen();   // listen for incoming TCP-connection
		DoNetworkStuff();                                      // handle network and easyWEB-stack
                                                           // events
		HTTPServer();
		UDP_SendHandler();
	}
#endif
}

// This function implements a very simple dynamic HTTP-server.
// It waits until connected, then sends a HTTP-header and the
// HTML-code stored in memory. Before sending, it replaces
// some special strings with dynamic values.
// NOTE: For strings crossing page boundaries, replacing will
// not work. In this case, simply add some extra lines
// (e.g. CR and LFs) to the HTML-code.

void HTTPServer(void)
{
  if (SocketStatus & SOCK_CONNECTED)             // check if somebody has connected to our TCP
  {
    if (SocketStatus & SOCK_DATA_AVAILABLE)      // check if remote TCP sent data
	{
	  // data in TCP_RX_BUF
      TCPReleaseRxBuffer();                      // and throw it away
	}

    if (SocketStatus & SOCK_TX_BUF_RELEASED)     // check if buffer is free for TX
    {
      if (!(HTTPStatus & HTTP_SEND_PAGE))        // init byte-counter and pointer to webside
      {                                          // if called the 1st time
        HTTPBytesToSend = sizeof(WebSide) - 1;   // get HTML length, ignore trailing zero
        PWebSide = (unsigned char *)WebSide;     // pointer to HTML-code
      }

      if (HTTPBytesToSend > MAX_TCP_TX_DATA_SIZE)     // transmit a segment of MAX_SIZE
      {
        if (!(HTTPStatus & HTTP_SEND_PAGE))           // 1st time, include HTTP-header
        {
          memcpy(TCP_TX_BUF, GetResponse, sizeof(GetResponse) - 1);
          memcpy(TCP_TX_BUF + sizeof(GetResponse) - 1, PWebSide, MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1);
          HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
          PWebSide += MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
        }
        else
        {
          memcpy(TCP_TX_BUF, PWebSide, MAX_TCP_TX_DATA_SIZE);
          HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE;
          PWebSide += MAX_TCP_TX_DATA_SIZE;
        }
          
        TCPTxDataCount = MAX_TCP_TX_DATA_SIZE;   // bytes to xfer
        InsertDynamicValues();                   // exchange some strings...
        TCPTransmitTxBuffer();                   // xfer buffer
      }
      else if (HTTPBytesToSend)                  // transmit leftover bytes
      {
        memcpy(TCP_TX_BUF, PWebSide, HTTPBytesToSend);
        TCPTxDataCount = HTTPBytesToSend;        // bytes to xfer
        InsertDynamicValues();                   // exchange some strings...
        TCPTransmitTxBuffer();                   // send last segment
        TCPClose();                              // and close connection
        HTTPBytesToSend = 0;                     // all data sent
      }

      HTTPStatus |= HTTP_SEND_PAGE;              // ok, 1st loop executed
    }
  }
  else
    HTTPStatus &= ~HTTP_SEND_PAGE;               // reset help-flag if not connected
}

void HTTPClient(void)
{
  if (SocketStatus & SOCK_CONNECTED)             // check if somebody has connected to our TCP
  {
    if (SocketStatus & SOCK_DATA_AVAILABLE)      // check if remote TCP sent data
	{
	  if (HTTPStatus == HTTPC_RECV_ACK)
	  {
	  	// answer
		HTTPStatus = HTTPC_FREE;
        TCPClose();                              // and close connection
	  }
	  // data in TCP_RX_BUF
      TCPReleaseRxBuffer();                      // and throw it away
	}

    if (SocketStatus & SOCK_TX_BUF_RELEASED)     // check if buffer is free for TX
    {
      if ((HTTPStatus == HTTPC_SEND_QUERY))
      {                                          // if called the 1st time
#if 0
		HTTPBytesToSend = sizeof(WebQuery) - 1;   // get HTML length, ignore trailing zero
		PWebSide = (unsigned char *)WebQuery;     // pointer to HTML-code
#else
		HTTPBytesToSend = strlen(HTTPQuery);
		PWebSide = (unsigned char *)HTTPQuery;
#endif
      }

      if (HTTPBytesToSend > 0) // 1st time, include HTTP-header
      {
        memcpy(TCP_TX_BUF, PWebSide, HTTPBytesToSend);
        TCPTxDataCount = HTTPBytesToSend;        // bytes to xfer
        TCPTransmitTxBuffer();                   // send last segment
        HTTPBytesToSend = 0;                     // all data sent

		HTTPStatus = HTTPC_RECV_ACK;
      }
    }
  }
}

// samples and returns the AD-converter value of channel 0

unsigned int GetAD7Val(void)
{
// Keil: function replaced to handle LPC2378 A/D converter.
  unsigned int val;

  AD0CR = 0x01000001 | 0x002E0400;       // Setup A/D: 10-bit AIN0 @ 3MHz
  do {
    val = AD0GDR;                        // Read A/D Data Register
  } while ((val & 0x80000000) == 0);     // Wait for end of A/D Conversion
  AD0CR &= ~0x01000001;                  // Stop A/D Conversion
  val = (val >> 6) & 0x03FF;             // Extract AIN0 Value
  return(val / 10);                      // result of A/D process 
}

// samples and returns AD-converter value of channel 1

unsigned int GetTempVal(void)
{
// Keil: function replaced to handle LPC2378 A/D converter.
  unsigned int val;

  AD0CR  = 0x01000002 | 0x002E0400;      // Setup A/D: 10-bit AIN1 @ 3MHz
  do {
    val = AD0GDR;                        // Read A/D Data Register
  } while ((val & 0x80000000) == 0);     // Wait for end of A/D Conversion
  AD0CR &= ~0x01000002;                  // Stop A/D Conversion
  val = (val >> 6) & 0x03FF;             // Extract AIN1 Value
  return(val / 10);                      // result of A/D process 
}

// searches the TX-buffer for special strings and replaces them
// with dynamic values (AD-converter results)

void InsertDynamicValues(void)
{
  unsigned char *Key;
           char NewKey[5];
  unsigned int i;
  
  if (TCPTxDataCount < 4) return;                     // there can't be any special string
  
  Key = TCP_TX_BUF;
  
  for (i = 0; i < (TCPTxDataCount - 3); i++)
  {
    if (*Key == 'A')
     if (*(Key + 1) == 'D')
       if (*(Key + 3) == '%')
         switch (*(Key + 2))
         {
           case '7' :                                 // "AD7%"?
           {
             sprintf(NewKey, "%3u", GetAD7Val());     // insert AD converter value
             memcpy(Key, NewKey, 3);                  // channel 7 (P6.7)
             break;
           }
           case 'A' :                                 // "ADA%"?
           {
             sprintf(NewKey, "%3u", GetTempVal());    // insert AD converter value
             memcpy(Key, NewKey, 3);                  // channel 10 (temp.-diode)
             break;
           }
         }
    Key++;
  }
}
