/**
	@file     I2C
	
	: NXP LPC1300
	: RealView MDK

	NUNCHUCK_SUPPORT -     NUNCKUCK.
	        .
	:   -    .

	@c	igorkov / 2011
 */
//#include <LPC13xx.H>
#include <LPC23xx.H>
#include <string.h>
#include "type.h"
#include "i2c.h"
#include "i2c-low.h"
//#include "timer.h"

#if defined( _LPC1700_ ) 
	#define I2C0SCLH    LPC_I2C0->I2SCLH
	#define I2C0SCLL    LPC_I2C0->I2SCLL
	#define I2C0CONSET  LPC_I2C0->I2CONSET
	#define I2C0CONCLR  LPC_I2C0->I2CONCLR
	#define I2C0STAT    LPC_I2C0->I2STAT
	#define I2C0DAT     LPC_I2C0->I2DAT
#elif defined( _LPC1300_ ) 
	#define I2C0SCLH    LPC_I2C->SCLH
	#define I2C0SCLL    LPC_I2C->SCLL
	#define I2C0CONSET  LPC_I2C->CONSET
	#define I2C0CONCLR  LPC_I2C->CONCLR
	#define I2C0STAT    LPC_I2C->STAT
	#define I2C0DAT     LPC_I2C->DAT
#else defined( _LPC237x_ ) 
	#define I2C0SCLH    I20SCLH
	#define I2C0SCLL    I20SCLL
	#define I2C0CONSET  I20CONSET
	#define I2C0CONCLR  I20CONCLR
	#define I2C0STAT    I20STAT
	#define I2C0DAT     I20DAT
#endif

/**
	I2C_Init
	  I2C
 */
void i2c_init_low(void)
{
    PCONP |= (1 << 19);
    PINSEL1 &= ~0x03C00000;
	PINSEL1 |= 0x01400000;	/* set PIO0.27 and PIO0.28 to I2C0 SDA and SCK */

	//LPC_SYSCON->PRESETCTRL |= (0x1<<1); // reset i2c
	//LPC_SYSCON->SYSAHBCLKCTRL |= (1<<5); // power on i2c

	//LPC_IOCON->PIO0_4 &= ~0x3F;	/*  I2C I/O config */
	//LPC_IOCON->PIO0_4 |=  0x01;		/* I2C SCL */
	//LPC_IOCON->PIO0_5 &= ~0x3F;	
	//LPC_IOCON->PIO0_5 |=  0x01;		/* I2C SDA */ 

	// I2C_freq = VPBUS / (SCLH+SCLL)
	I2C0SCLH		= I2C_DIV_H;
	I2C0SCLL		= I2C_DIV_L;
	
	// disable and reset interface
	I2C0CONCLR = 0xFF;

	// enable interface
	I2C0CONSET = (1<<I2CON_I2EN);

	return;
}

int i2c_write(uint8_t dev_addr, uint8_t addr, uint8_t *data, int len)
{
	return i2c_io((dev_addr<<1), addr, data, len, I2C_WRITE);
}

int i2c_read(uint8_t dev_addr, uint8_t addr, uint8_t *data, int len)
{
	return i2c_io((dev_addr<<1), addr, data, len, I2C_READ);
}

/**
	  / I2C .
	
	@param	[in]	Addr		   
	@param	[in]	Data		  
	@param	[in]	Num			- / 
	@param	[in]	WriteBit	, /
 */
int i2c_io(unsigned char bDevAddr, unsigned char bAddr, unsigned char *abData, unsigned char bNum, unsigned char bWriteBit)
{
	unsigned char abTmp[64];
	int i = 0;
	unsigned char bI2CError = 0;

	abTmp[0] = bAddr;

	if (bNum > 64-1)
		return I2C_OVF_BUFFER;

	if (bWriteBit)
	{
		memcpy(&abTmp[1], abData, bNum);
		while((bI2CError = i2cMasterSendNI(bDevAddr, bNum+1, abTmp)) != I2C_OK)
		{
			if (i++ > 10) // 10    
			{
				goto __break;
			}
		}
		if (bI2CError == 0)
			i2cSendStop();
	}
	else
	{
		while((bI2CError = i2cMasterSendNI(bDevAddr, 1, abTmp)) != I2C_OK)
		{
			if (i++ > 10) // 10    
			{
				goto __break;
			}
		}
		bI2CError = i2cMasterReceiveNI(bDevAddr, bNum, abData);
	}
__break:

	return bI2CError;
}

// ----------------------------------- LOW ----------------------------------- 

void i2cSendStart(void)
{
	I2C0CONSET = BIT(I2CON_STA);
	I2C0CONCLR = BIT(I2CON_SI);
}

void i2cSendStop(void)
{
	int n;
	I2C0CONSET = BIT(I2CON_STO);
	I2C0CONCLR = BIT(I2CON_SI);
	for (n = 0; n < 10000; n++) {  }
	I2C0CONCLR = 0xFF;
	// enable interface
	I2C0CONSET = (1<<I2CON_I2EN);
}

int i2cWaitForComplete(void)
{
	int i = 0;
	// wait for a valid status code
	while (I2C0STAT == I2C_NO_INFO)
	{
		i++;
		if (i > 0x20000) //  
		{
			i2cSendStop();
			return I2C_ERROR_NODEV;
		}
	}
	return I2C_OK;
}

void i2cSendByte(unsigned char data)
{
	// save data into data register
	I2C0DAT = data;
	// clear SI bit to begin transfer
	I2C0CONCLR = BIT(I2CON_SI);
}

void i2cReceiveByte(unsigned char ackFlag)
{
	// begin receive over i2c
	if( ackFlag )
	{
		// ackFlag = TRUE: ACK the recevied data
		I2C0CONSET = BIT(I2CON_AA);
	}
	else
	{
		// ackFlag = FALSE: NACK the recevied data
		I2C0CONCLR = BIT(I2CON_AA);
	}
	// clear SI bit to begin transfer
	I2C0CONCLR = BIT(I2CON_SI);
}

unsigned char i2cGetReceivedByte(void)
{
	return I2C0DAT;
}

unsigned char i2cGetStatus(void)
{
	return I2C0STAT;
}

unsigned char i2cMasterSendNI(unsigned char deviceAddr, unsigned char length, unsigned char * data)
{
	unsigned char retval = I2C_OK;
	unsigned char count = length;

	// send start condition
	i2cSendStart();
	if (i2cWaitForComplete())
		return I2C_ERROR;

#if defined( _LPC214x_ ) || defined( _LPC237x_ )
	//I2C0CONCLR = BIT(I2CON_STA);
#endif

	// save data into data register
	I2C0DAT = deviceAddr & 0xFE;
	// clear SI bit to begin transfer
	I2C0CONCLR = BIT(I2CON_SI) | BIT(I2CON_STA);

	if (i2cWaitForComplete())
		return I2C_ERROR_NODEV;

	// check if device is present and live
	if (I2C0STAT == I2C_MT_SLA_ACK)
	{
		// send data
		while(count)
		{
			i2cSendByte( *data++ );
			if (i2cWaitForComplete())
				return I2C_ERROR;
			count--;
		}
	}
	else
	{
		// device did not ACK it's address,
		// data will not be transferred
		// return error
		retval = I2C_ERROR_NODEV;
	}

	if (/*length != 1 ||*/ retval) // add igorkov
	{
		// transmit stop condition
		// leave with TWEA on for slave receiving
		i2cSendStop();
	}

	return retval;
}

unsigned char i2cMasterReceiveNI(unsigned char deviceAddr, unsigned char length, unsigned char *data)
{
	unsigned char retval = I2C_OK;

	// send start condition
	i2cSendStart();
	if (i2cWaitForComplete())
		return I2C_ERROR;

#if defined( _LPC214x_ ) || defined( _LPC237x_ )
	I2C0CONCLR = BIT(I2CON_STA);
#endif

	// save data into data register
	I2C0DAT = deviceAddr | 0x01;
	// clear SI bit to begin transfer
	I2C0CONCLR = BIT(I2CON_SI) | BIT(I2CON_STA);

	if (i2cWaitForComplete())
		return I2C_ERROR;

	// check if device is present and live
	if( I2C0STAT == I2C_MR_SLA_ACK)
	{
		// accept receive data and ack it
		while(length > 1)
		{
			i2cReceiveByte(TRUE);
			if (i2cWaitForComplete())
				return I2C_ERROR;
			*data++ = i2cGetReceivedByte();
			// decrement length
			length--;
		}
		
		// accept receive data and nack it (last-byte signal)
		i2cReceiveByte(FALSE);
		if (i2cWaitForComplete())
			return I2C_ERROR;
		*data++ = i2cGetReceivedByte();
	}
	else
	{
		// device did not ACK it's address,
		// data will not be transferred
		// return error
		retval = I2C_ERROR;
	}

	// transmit stop condition
	// leave with TWEA on for slave receiving
	i2cSendStop();

	return retval;
}
