#ifndef _I2C_H_
#define _I2C_H_

#include <stdint.h>

/**
	LPC2100
	I2C_freq = VPBUS / (SCLH+SCLL) = 60MHz / (80 + 80) = 375kHz
	LPC2300
	I2C_freq = VPBUS / (SCLH+SCLL) = 60MHz / (80 + 80) = 375kHz
	LPC1700:
	I2C_freq = VPBUS / (SCLH+SCLL) = 48MHz / (80 + 80) = 300kHz
	LPC1300:
	I2C_freq = VPBUS / (SCLH+SCLL) = 72MHz / (360 + 360) = 100kHz
	I2C_freq = VPBUS / (SCLH+SCLL) = 72MHz / (180 + 180) = 200kHz
	I2C_freq = VPBUS / (SCLH+SCLL) = 72MHz / (90 + 90) = 400kHz
 */
//#define I2C_DIV_H 360
//#define I2C_DIV_L 360
//#define I2C_DIV_H 180
//#define I2C_DIV_L 180
#define I2C_DIV_H 90
#define I2C_DIV_L 90

#define I2C_WRITE 1
#define I2C_READ  0

void i2c_init_low(void);
int i2c_io(unsigned char dev_addr, unsigned char addr, unsigned char *data, unsigned char num, unsigned char write_bit);
int i2c_is_error(void);

int i2c_write(uint8_t dev_addr, uint8_t addr, uint8_t *buffer, int len);
int i2c_read(uint8_t dev_addr, uint8_t addr, uint8_t *buffer, int len);

#define I2C_OK              0x00
#define I2C_ERROR_NODEV     0x01
#define I2C_ERROR           0x02
#define I2C_OVF_BUFFER      0x03


// ------------------------------ low ------------------------------------------

/* Master */
#define I2C_START					0x08
#define I2C_REP_START				0x10
/* Master Transmitter */
#define I2C_MT_SLA_ACK				0x18
#define I2C_MT_SLA_NACK				0x20
#define I2C_MT_DATA_ACK				0x28
#define I2C_MT_DATA_NACK			0x30
#define I2C_MT_ARB_LOST				0x38
/* Master Receiver */
#define I2C_MR_ARB_LOST				0x38
#define I2C_MR_SLA_ACK				0x40
#define I2C_MR_SLA_NACK				0x48
#define I2C_MR_DATA_ACK				0x50
#define I2C_MR_DATA_NACK			0x58
/* Misc */
#define I2C_NO_INFO					0xF8
#define I2C_BUS_ERROR				0x00
/* bit defines */
#define I2CON_AA			2
#define I2CON_SI			3
#define I2CON_STO			4
#define I2CON_STA			5
#define I2CON_I2EN			6

// Low-level I2C transaction commands 
//! Send an I2C start condition in Master mode
void i2cSendStart(void);
//! Send an I2C stop condition in Master mode
void i2cSendStop(void);
//! Wait for current I2C operation to complete
int i2cWaitForComplete(void);
//! Send an (address|R/W) combination or a data byte over I2C
void i2cSendByte(unsigned char data);
//! Receive a data byte over I2C  
// ackFlag = TRUE if recevied data should be ACK'ed
// ackFlag = FALSE if recevied data should be NACK'ed
void i2cReceiveByte(unsigned char ackFlag);
//! Pick up the data that was received with i2cReceiveByte()
unsigned char i2cGetReceivedByte(void);
//! Get current I2c bus status from TWSR
unsigned char i2cGetStatus(void);

//! send I2C data to a device on the bus (non-interrupt based)
unsigned char i2cMasterSendNI(unsigned char deviceAddr, unsigned char length, unsigned char * data);
//! receive I2C data from a device on the bus (non-interrupt based)
unsigned char i2cMasterReceiveNI(unsigned char deviceAddr, unsigned char length, unsigned char *data);

// some handy defines
#ifndef BIT
#define BIT(n)				(1 << (n))
#endif

#endif /* _I2C_H_ */
