/******************************************************************
 *****                                                        *****
 *****  Name: tcpip.c                                         *****
 *****  Ver.: 1.0                                             *****
 *****  Date: 07/05/2001                                      *****
 *****  Auth: Andreas Dannenberg                              *****
 *****        HTWK Leipzig                                    *****
 *****        university of applied sciences                  *****
 *****        Germany                                         *****
 *****  Func: implements the TCP/IP-stack and provides a      *****
 *****        simple API to the user                          *****
 *****                                                        *****
 ******************************************************************/

#include "tcpip.h"
#include "EMAC.h"         // Keil: Line added
#include "udp.h"
#include <string.h>       // Keil: Line added
#include <LPC23xx.h>      // Keil: Register definition file for LPC2378


#define COUNTER_MAX 0x000fffffUL

//static unsigned char bReady = 0;
static unsigned long nCounter = 0;
unsigned short UDP_CRC16(unsigned short *pData, unsigned short nLen)
{
#if 0
	unsigned long nCRC = 0L;
	while(nLen > 1)
	{
		nCRC += *pData++;
		nLen -= sizeof(*pData);
	}
	if(nLen) nCRC += *(unsigned char *)pData;
	nCRC = (nCRC >> 16) + (nCRC & 0xFFFF);
	nCRC += (nCRC >> 16);
	return (unsigned short)nCRC;
#else //0
	return 0;
#endif //0
}

void SendUDP(unsigned short nPortSrc, //source port 
			 unsigned short nPortDst, //destination port
			 unsigned short *pData, //data
			 unsigned short nDataLength //data length (bytes)
			 )
{
	unsigned short nDataLengthWords = (nDataLength + 1) & (~1);
	// Ethernet
	memcpy(&TxFrame2[ETH_DA_OFS], &RecdFrameMAC, 6);
	memcpy(&TxFrame2[ETH_SA_OFS], &MyMAC, 6);
	*(unsigned short *)&TxFrame2[ETH_TYPE_OFS] = SWAPB(FRAME_IP);
	
	// IP   
	*(unsigned short *)&TxFrame2[IP_VER_IHL_TOS_OFS] = SWAPB(IP_VER_IHL);
	WriteWBE(&TxFrame2[IP_TOTAL_LENGTH_OFS], IP_HEADER_SIZE + UDP_HEADER_SIZE + nDataLengthWords);
	*(unsigned short *)&TxFrame2[IP_IDENT_OFS] = 0;
	*(unsigned short *)&TxFrame2[IP_FLAGS_FRAG_OFS] = 0;
	*(unsigned short *)&TxFrame2[IP_TTL_PROT_OFS] = SWAPB((DEFAULT_TTL << 8) | PROT_UDP);
	*(unsigned short *)&TxFrame2[IP_HEAD_CHKSUM_OFS] = 0;
	memcpy(&TxFrame2[IP_SOURCE_OFS], &MyIP, 4);
	//memcpy(&TxFrame2[IP_DESTINATION_OFS], &RecdFrameIP, 4);
	*(unsigned char *)&TxFrame2[IP_DESTINATION_OFS + 0] = REMIP_1;
	*(unsigned char *)&TxFrame2[IP_DESTINATION_OFS + 1] = REMIP_2;
	*(unsigned char *)&TxFrame2[IP_DESTINATION_OFS + 2] = REMIP_3;
	*(unsigned char *)&TxFrame2[IP_DESTINATION_OFS + 3] = REMIP_4;
	*(unsigned short *)&TxFrame2[IP_HEAD_CHKSUM_OFS] = CalcChecksum(&TxFrame2[IP_VER_IHL_TOS_OFS], IP_HEADER_SIZE, 0);

	//UDP
	*(unsigned short *)&TxFrame2[UDP_SRCPORT_OFS] = SWAPB(nPortSrc);
	*(unsigned short *)&TxFrame2[UDP_DSTPORT_OFS] = SWAPB(nPortDst);
	*(unsigned short *)&TxFrame2[UDP_DATALEN_OFS] = SWAPB(nDataLengthWords + UDP_HEADER_SIZE);
	*(unsigned short *)&TxFrame2[UDP_CRC_OFS] = 0;
	memcpy(&TxFrame2[UDP_DATA_OFS], pData, nDataLength);
	if(nDataLength & 1) TxFrame2[UDP_DATA_OFS + nDataLength] = 0;

	TxFrame2Size = ETH_HEADER_SIZE + IP_HEADER_SIZE + UDP_HEADER_SIZE + nDataLengthWords;
	TransmitControl |= SEND_FRAME2;

	//bReady = 1;
}

unsigned char aData2Recv[UDP_HEADER_SIZE + sizeof(struct tag_DataExample) + sizeof(struct tag_MSFS_FSUIPC3_Data_Chunk) + 1];
unsigned char * pDataRecv = aData2Recv;
unsigned short nRecvBufferSize = sizeof(aData2Recv);
unsigned char * pRecvDataBuffer;

unsigned char aData2Send[sizeof(struct tag_DataExample) + sizeof(struct tag_MSFS_FSUIPC3_Data_Chunk)];

void UDP_InitReceiver(void)
{
	if(((WORD)pDataRecv) & 1) {pDataRecv++; nRecvBufferSize--;}//even aling pointer
	pRecvDataBuffer = pDataRecv + UDP_HEADER_SIZE;
	nRecvBufferSize &= ~1;
}

#ifdef UDP_ENABLE_DEBUG_INC
	void UDP_Debug_IncCounter(void)
	{
		((PDataExample)aData2Send)->dwHeaderVersion++;
	}
#endif //UDP_ENABLE_DEBUG_INC

void UDP_Reciever(void)
{
	CopyFromFrame_EMAC(pDataRecv, nRecvBufferSize);
	//UDP_Debug_IncCounter();
	//((PDataExample)aData2Send)->dwHeaderVersion = (DWORD)UDP_GetRecvDataChunk()->uData.nData;
}

PDataExample UDP_GetRecvDataHeader(void)
{
	return (PDataExample)pRecvDataBuffer;
}
PMSFS_FSUIPC3_Data_Chunk UDP_GetRecvDataChunk(void)
{
	return (PMSFS_FSUIPC3_Data_Chunk)&pRecvDataBuffer[sizeof(struct tag_DataExample)];
}
	 




void UDP_Init_MSFS_Array(void)
{
	PDataExample pHeader = (PDataExample)aData2Send;
	PMSFS_FSUIPC3_Data_Chunk pChunk = (PMSFS_FSUIPC3_Data_Chunk)&aData2Send[sizeof(struct tag_DataExample)];
	
	memset(pHeader, 0x00, sizeof(*pHeader));
	//TODO: fill all (!) fields!!! 
	//pHeader->dwHeaderVersion = 0x01020304;//header vererion
    pHeader->dwHeaderLen = sizeof(*pHeader);//header length (bytes) & flags
    pHeader->dwDeviceID = 0x00;//device ID check this from server
	pHeader->dwLiveTime = 10000;
	pHeader->dwDataLength = sizeof(struct tag_MSFS_FSUIPC3_Data_Chunk);
	//
	memset(pChunk, 0x00, sizeof(*pChunk));
	pChunk->wAddress = 0x02BC; //Air Speed, as knots * 128
	pChunk->wSizeInBytes = 4;
}


void UDP_SendHandler(void)
{
	while(nCounter-- == 0)
	{
		nCounter = COUNTER_MAX;
		SendUDP(9009, 9009, (unsigned short *)aData2Send, sizeof(aData2Send));
	}	
}

